const browserAPI = typeof browser !== 'undefined' ? browser : chrome;

let serverUrl = "http://localhost:5000/submit";
let password = "";
let clientId = null;
let isRecording = false;
let checkInterval = null;
let currentSessionId = null;


function loadSettings() {
    return new Promise((resolve) => {
        if (typeof browser !== 'undefined') {
            browserAPI.storage.sync.get(["serverUrl", "password"])
                .then((result) => {
                    processSettings(result);
                    resolve();
                })
                .catch((error) => {
                    console.error("Ошибка загрузки настроек:", error);
                    resolve();
                });
        } else {
            browserAPI.storage.sync.get(["serverUrl", "password"], (result) => {
                if (chrome.runtime.lastError) {
                    console.error("Ошибка загрузки настроек:", chrome.runtime.lastError);
                } else {
                    processSettings(result);
                }
                resolve();
            });
        }
    });
}

function processSettings(result) {
    if (result.serverUrl) {
        serverUrl = result.serverUrl;
        console.log("URL сервера загружен из настроек:", serverUrl);
    }
    if (result.password) {
        password = result.password;
        console.log("Пароль загружен из настроек.");
    }
}

function handleClientId() {
    if (typeof browser !== 'undefined') {
        browserAPI.storage.sync.get("clientId")
            .then((result) => {
                if (result.clientId) {
                    clientId = result.clientId;
                    console.log("ID клиента загружен:", clientId);
                } else {
                    generateAndSaveId();
                }
            })
            .catch(console.error);
    } else {
        browserAPI.storage.sync.get("clientId", (result) => {
            if (chrome.runtime.lastError) {
                console.error("Ошибка загрузки ID клиента:", chrome.runtime.lastError);
            } else if (result.clientId) {
                clientId = result.clientId;
                console.log("ID клиента загружен:", clientId);
            } else {
                generateAndSaveId();
            }
        });
    }
}

function generateUUID() {
    return 'xxxxxxxx-xxxx-4xxx-yxxx-xxxxxxxxxxxx'.replace(/[xy]/g, (c) => {
        const r = (Math.random() * 16) | 0;
        const v = c === 'x' ? r : (r & 0x3) | 0x8;
        return v.toString(16);
    });
}

function generateAndSaveId() {
    clientId = generateUUID();
    if (typeof browser !== 'undefined') {
        browserAPI.storage.sync.set({ clientId })
            .then(() => console.log("ID клиента создан и сохранен:", clientId))
            .catch(console.error);
    } else {
        browserAPI.storage.sync.set({ clientId }, () => {
            if (chrome.runtime.lastError) {
                console.error("Ошибка сохранения ID клиента:", chrome.runtime.lastError);
            } else {
                console.log("ID клиента создан и сохранен:", clientId);
            }
        });
    }
}


async function checkSessionStatus() {
    try {
        const statusUrl = 'http://127.0.0.1:5000/api/session_status';
        const response = await fetch(statusUrl, {
            headers: { 'X-Password': password }
        });
        
        const { active, session_id } = await response.json();
        
        if (active && !isRecording) {
            startRecording(session_id);
        } else if (!active && isRecording) {
            stopRecording();
        }
    } catch (error) {
        console.error('Ошибка проверки сессии:', error);
    }
}

function startRecording(sessionId) {
    isRecording = true;
    currentSessionId = sessionId;
    console.log('Запись действий начата для сессии:', sessionId);
}

function stopRecording() {
    isRecording = false;
    currentSessionId = null;
    console.log('Запись действий остановлена');
}


function createEventJSON(eventType, data) {
    const timeStamp = new Date().toISOString();
    const pageUrl = window.location.href;
    return {
        "@version": 1,
        "type": `browserext.${eventType}`,
        "TimeStamp": timeStamp,
        "data": {...data, clientId, pageUrl},
        "@timestamp": timeStamp
    };
}


async function sendDataToServer(eventData) {
    if (!isRecording || !serverUrl || !password) return;

    try {
        const response = await fetch(serverUrl, {
            method: 'POST',
            headers: {
                'Content-Type': 'application/json',
                'X-Password': password,
            },
            body: JSON.stringify({
                ...eventData,
                session_id: currentSessionId
            })
        });

        const responseData = await response.json();
        if (response.ok) {
            console.log("Данные успешно отправлены на сервер:", responseData);
        } else {
            console.error("Ошибка при отправке данных на сервер:", response.status, responseData);
        }
    } catch (error) {
        console.error("Ошибка подключения к серверу:", error);
    }
}

function debounce(func, delay) {
    let timeout;
    return function(...args) {
        clearTimeout(timeout);
        timeout = setTimeout(() => func(...args), delay);
    };
}

async function initialize() {
    await loadSettings();
    handleClientId();

    checkInterval = setInterval(checkSessionStatus, 5000);
    checkSessionStatus();


    document.addEventListener("keydown", (event) => {
        if (!isRecording) return;
        const key = event.key;
        const code = event.code;
        let description;

        if (key.match(/^\p{Number}$/u)) {
            description = "number";
        } else if (key.match(/^\p{Letter}$/u)) {
            description = "letter";
        } else if (key.match(/^\p{Punctuation}$|^\p{Symbol}$/u)) {
            description = "symbol";
        } else {
            description = `Клавиша: ${key} Код: ${code}`;
        }

        const eventData = createEventJSON('keydown', { description });
        console.log("Событие keydown:", JSON.stringify(eventData));
        sendDataToServer(eventData);
    });


    document.addEventListener("mousedown", (event) => {
        if (!isRecording) return;
        const eventData = createEventJSON('mousedown', { button: event.button, description: `Клик мышью: ${event.button}` });
        console.log("Событие mousedown:", JSON.stringify(eventData));
        sendDataToServer(eventData);
    });

    let accumulatedPixels = 0;

    const handleWheel = debounce((event) => {
        if (!isRecording) return;
        const direction = accumulatedPixels > 0 ? "вниз" : "вверх";
        const pixels = Math.abs(accumulatedPixels);

        const eventData = createEventJSON('wheel', { direction, pixels: `${pixels}px`, description: "Прокрутка колёсика мыши"});
        console.log("Событие wheel:", JSON.stringify(eventData));
        sendDataToServer(eventData);

        accumulatedPixels = 0;
    }, 200);

    document.addEventListener("wheel", (event) => { if (!isRecording) return; accumulatedPixels += event.deltaY; handleWheel(event);});

    document.addEventListener("click", (event) => {
        if (!isRecording) return;
        const eventType = 'click';
        const clickedElement = event.target;
        const tag = clickedElement.tagName;
        const id = clickedElement.id || "отсутствует";
        const className = clickedElement.className || "отсутствует";

        let description = "Клик по элементу";
        let data = { tag, id, className };

        if (tag === 'A') {
            const href = clickedElement.href;
            data.href = href;
            description = "Клик по ссылке";

            localStorage.setItem('clickedElementInfo', JSON.stringify(data));
            navigateWithPromise(href);
        } else if (tag === 'INPUT') {
            const placeholder = clickedElement.placeholder || "отсутствует";
            data.placeholder = placeholder;
            description = "Клик по полю ввода";
        }

        data.description = description;

        const eventData = createEventJSON(eventType, data);
        console.log("Событие click:", JSON.stringify(eventData));
        sendDataToServer(eventData);
    });


    function navigateWithPromise(href) {
        return new Promise((resolve) => {
            window.location.href = href;
            window.addEventListener('load', resolve);
        });
    }


    document.addEventListener("copy", (event) => {
        if (!isRecording) return;
        const selectedText = window.getSelection().toString();

        let outputText = selectedText.length > 50 ? selectedText.substring(0, 49) + '…' : selectedText;

        if (selectedText.length > 0) {
            const eventData = createEventJSON('copy', { selectedText: outputText, description: "Скопирован текст" });
            console.log("Событие copy:", JSON.stringify(eventData));
            sendDataToServer(eventData);
        }
    });
}

initialize();